#include <bits/stdc++.h>
#include "testlib.h"

using namespace std;

const int MIN_M = 1, MAX_M = 100'000;
const int MIN_N = 2;
//                           0  1   2      3      4      5
const array<int, 6> MAX_N = {5, 10, MAX_M, MAX_M, MAX_M, MAX_M};

void VisitVertexes(int current, const vector<set<int>>& graph, vector<bool>& visited) {
    visited[current] = true;
    for (int next : graph[current]) {
        if (!visited.at(next)) {
            VisitVertexes(next, graph, visited);
        }
    }
}

bool IsConnected(const vector<set<int>>& graph) {
    vector<bool> visited(graph.size(), false);
    VisitVertexes(0, graph, visited);
    
    for (bool was_visited : visited) {
        if (!was_visited) {
            return false;
        }
    }
    return true;
}

bool HasPath1ToN(const vector<set<int>>& graph, int n) {
    for (int i = 0; i < n - 1; ++i) {
        if (!graph[i].count(i + 1)) {
            return false;
        }
    }
    return true;
}

bool IsBipartite(const vector<set<int>>& g, vector<char>* ans_color = nullptr) {
    vector<char> color(g.size(), -1);
    for (size_t to_check = 0; to_check < g.size(); ++to_check) {
        if (color[to_check] == -1) {
            color[to_check] = 0;
            queue<int> bfs;
            bfs.push(to_check);
            while (!bfs.empty()) {
                int cur = bfs.front();
                bfs.pop();
                for (int nxt : g[cur]) {
                    if (color[nxt] == color[cur]) {
                        return false;
                    }
                    if (color[nxt] == -1) {
                        color[nxt] = 1 - color[cur];
                        bfs.push(nxt);
                    }
                }
            }
        }
    }
    if (ans_color != nullptr) {
        *ans_color = std::move(color);
    }
    return true;
}

int main(int argc, char* argv[]) {
    registerValidation(argc, argv);
    int group = atoi(validator.group().data());
	ensuref(0 <= group && group <= 5, "Incorrect test group %d", group);
    
    int n = inf.readInt(MIN_N, MAX_N[group], "n");
    inf.readSpace();
    int m = inf.readInt(MIN_M, MAX_M, "m");
    if (group == 1 || group == 2) {
        ensure(m == n - 1);
    }
    inf.readEoln();
    
    vector<set<int>> graph(n);
    for (int i = 0; i < m; ++i) {
        int u = inf.readInt(1, n, "u");
        inf.readSpace();
        int v = inf.readInt(1, n, "v");
        inf.readEoln();
        ensuref(u != v, "Got loop for vertex %d", u);
        
        --u; --v;
        ensuref(!graph[u].count(v) && !graph[v].count(u), "Got parallel edge between %d and %d", u, v);
        graph[u].insert(v);
        graph[v].insert(u);
        if (group == 4 && i < n - 1) {
            ensuref(u == i && v == i + 1, "group 4");
        }
    }
    ensure(IsConnected(graph));
    if (group == 3) {
        ensure(IsBipartite(graph));
    }
    inf.readEof();
}
